﻿using System;

namespace EnigmaLib
{
    public interface IUiEnigmaMachine
    {
        IUiPlugboard Plugboard { get; }
        /// <summary>Normal = true. Set to false to enter cryptoanalysis mode.</summary>
        bool AutoAdvance { get; set; }
        int GetRotorCount();
        void SetRotorCount(int count);
        void SetReflector(Reflector reflector);
        void RemoveReflector();
        /// <summary>Get rotor. 1-based index</summary>
        /// <param name="position"></param>
        Rotor GetRotor(int position);
        /// <summary>Get the 1-based position of a rotor</summary>
        /// <param name="rotor"></param>
        /// <returns></returns>
        int GetPosition(Rotor rotor);
        /// <summary>Set rotor. 1-based index</summary>
        /// <param name="position"></param>
        /// <param name="rotor"></param>
        void SetRotor(int position, Rotor rotor);
        void RemoveRotor(Rotor rotor);

        char Encrypt(char c);

        /// <summary>Reverse 1 step. Only works if AutoAdvance = true.
        /// Note: reversing won't always result in the letters looking exactly like they
        /// did before the reverse, but since the machine is stepped before encryption,
        /// the encryption will still be correct</summary>
        void Reverse();

        /// <summary>Reset to starting position</summary>
        void Reset();

        event RotorChangedEventHandler RotorAdded;
        event RotorChangedEventHandler RotorRemoved;
        event ReflectorChangedEventHandler ReflectorAdded;
        event ReflectorChangedEventHandler ReflectorRemoved;
        event EventHandler AutoAdvanceChanged;
        event EventHandler RotorCountChanged;
    }
}
