using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using EnigmaLib;
using HistoricalEnigma;

namespace EnigmaWinForm
{
    public partial class Form1 : Form
    {
        EnigmaMachine _enigma = new EnigmaMachine(3);

        public Form1()
        {
            InitializeComponent();

            // must be set before setting rotors
            reflectorCb.SelectedIndex = 0;

            _rotorBoxes = new ComboBox[] { rotor1Cb, rotor2Cb, rotor3Cb };

            rotor4Cb.SelectedIndex = 0;
            rotor1Cb.SelectedIndex = 0;
            rotor2Cb.SelectedIndex = 0;
            rotor3Cb.SelectedIndex = 0;

            return;

            // test values
            rotor1Cb.SelectedIndex = 2;
            rotor2Cb.SelectedIndex = 4;
            rotor3Cb.SelectedIndex = 1;
            ring1Tb.Text = "N";
            ring2Tb.Text = "A";
            ring3Tb.Text = "S";
            start1Tb.Text = "X";
            start2Tb.Text = "O";
            start3Tb.Text = "M";
            plugsTb.Text = "QG AI NW SB EU XL DZ OR";
            ring1Tb_Leave(null, null);
            ring2Tb_Leave(null, null);
            ring3Tb_Leave(null, null);
            start1Tb_Leave(null, null);
            start2Tb_Leave(null, null);
            start3Tb_Leave(null, null);
            plugsTb_Leave(null, null);
            inputTb.Text = "dinah is the coolest spy";
        }

        private void reflectorCb_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_enigma.GetRotorCount() == 3)
            {
                if (reflectorCb.SelectedItem.ToString().ToLower() == "b")
                {
                    _enigma.SetReflector(HistoryFactory.ReflectorB());
                }
                else // c
                {
                    _enigma.SetReflector(HistoryFactory.ReflectorC());
                }
            }
            else // _enigma.GetRotorCount() == 4
            {
                if (reflectorCb.SelectedItem.ToString().ToLower() == "b")
                {
                    _enigma.SetReflector(HistoryFactory.ReflectorBThin());
                }
                else // c
                {
                    _enigma.SetReflector(HistoryFactory.ReflectorCThin());
                }
            }
            encrypt();
        }

        private void rotor4Cb_SelectedIndexChanged(object sender, EventArgs e)
        {
            string r = rotor4Cb.SelectedItem.ToString();
            r = r.ToLower();
            Rotor rotor = null;
            switch (r)
            {
                case "beta":
                    rotor = HistoryFactory.RotorBeta();
                    break;
                case "gamma":
                    rotor = HistoryFactory.RotorGamma();
                    break;
            }

            if (rotor == null)
            {
                _enigma.SetRotorCount(3);
                ring4Tb.Text = "";
                ring4Tb_Leave(sender, e);
                start4Tb.Text = "";
                start4Tb_Leave(sender, e);
                enigma4Tb.Text = "";
            }
            else
            {
                _enigma.SetRotorCount(4);
                _enigma.SetRotor(4, rotor);
                ring4Tb.Text = "A";
                ring4Tb_Leave(sender, e);
                start4Tb.Text = "A";
                start4Tb_Leave(sender, e);
            }

            //encrypt(); do at end of reflector's method
            reflectorCb_SelectedIndexChanged(sender, e);
        }
        private void rotor1Cb_SelectedIndexChanged(object sender, EventArgs e)
        {
            limitBoxes(rotor1Cb);
            string r = rotor1Cb.SelectedItem.ToString();
            _enigma.SetRotor(3, getRotor(r));
            ring1Tb.Text = "A";
            ring1Tb_Leave(sender, e);
            start1Tb.Text = "A";
            start1Tb_Leave(sender, e);
            encrypt();
        }
        private void rotor2Cb_SelectedIndexChanged(object sender, EventArgs e)
        {
            limitBoxes(rotor2Cb);
            string r = rotor2Cb.SelectedItem.ToString();
            _enigma.SetRotor(2, getRotor(r));
            ring2Tb.Text = "A";
            ring2Tb_Leave(sender, e);
            start2Tb.Text = "A";
            start2Tb_Leave(sender, e);
            encrypt();
        }
        private void rotor3Cb_SelectedIndexChanged(object sender, EventArgs e)
        {
            limitBoxes(rotor3Cb);
            string r = rotor3Cb.SelectedItem.ToString();
            _enigma.SetRotor(1, getRotor(r));
            ring3Tb.Text = "A";
            ring3Tb_Leave(sender, e);
            start3Tb.Text = "A";
            start3Tb_Leave(sender, e);
            encrypt();
        }
        Rotor getRotor(string r)
        {
            r = r.ToLower();
            switch (r)
            {
                case "ii":
                    return HistoryFactory.RotorII();
                case "iii":
                    return HistoryFactory.RotorIII();
                case "iv":
                    return HistoryFactory.RotorIV();
                case "v":
                    return HistoryFactory.RotorV();
                case "vi":
                    return HistoryFactory.RotorVI();
                case "vii":
                    return HistoryFactory.RotorVII();
                case "viii":
                    return HistoryFactory.RotorVIII();
                default:
                case "i":
                    return HistoryFactory.RotorI();
            }
        }
        string[] _standardRotors = new string[] {
            "I", "II", "III", "IV", "V", "VI", "VII", "VIII" };
        ComboBox[] _rotorBoxes;
        void limitBoxes(ComboBox cb)
        {
            int cbIndex;
            for (cbIndex = 0; cbIndex < _rotorBoxes.Length; cbIndex++)
                if (_rotorBoxes[cbIndex] == cb)
                    break;
            for (int i = 0; i < _rotorBoxes.Length; i++)
            {
                // don't evaluate self
                if (_rotorBoxes[i] == cb)
                    continue;
                // otherIndex: if the changed box is 1 and the box in this
                // iteration of the for loop is 0, then the 'other' box is 2
                int otherIndex = 3 - i - cbIndex;
                List<string> rList = new List<string>(_standardRotors);
                rList.Remove(cb.SelectedItem.ToString());
                if (_rotorBoxes[i].Items.Contains(cb.SelectedItem.ToString()))
                    _rotorBoxes[i].Items.Remove(cb.SelectedItem.ToString());
                if (_rotorBoxes[otherIndex].SelectedItem != null
                    && rList.Contains(_rotorBoxes[otherIndex].SelectedItem.ToString()))
                {
                    rList.Remove(_rotorBoxes[otherIndex].SelectedItem.ToString());
                }
                if (_rotorBoxes[i].Items.Contains(
                    _rotorBoxes[otherIndex].SelectedIndex.ToString()))
                {
                    _rotorBoxes[i].Items.Remove(_rotorBoxes[otherIndex].SelectedIndex.ToString());
                }
                foreach (string r in rList)
                    if (!_rotorBoxes[i].Items.Contains(r))
                        _rotorBoxes[i].Items.Add(r);
            }
        }

        private void ring4Tb_Leave(object sender, EventArgs e)
        {
            if (_enigma.GetRotorCount() == 3)
                return;
            setRing(4, ring4Tb.Text);
        }
        private void ring1Tb_Leave(object sender, EventArgs e)
        {
            setRing(3, ring1Tb.Text);
        }
        private void ring2Tb_Leave(object sender, EventArgs e)
        {
            setRing(2, ring2Tb.Text);
        }
        private void ring3Tb_Leave(object sender, EventArgs e)
        {
            setRing(1, ring3Tb.Text);
        }
        private void setRing(int position, string t)
        {
            Rotor r = _enigma.GetRotor(position);
            try
            {
                r.SetRing(Convert.ToChar(t));
                encrypt();
            }
            catch
            {
                messageTb.Text = (t ?? "[null]") + " is an invalid ring setting. ";
            }
        }

        private void start4Tb_Leave(object sender, EventArgs e)
        {
            if (_enigma.GetRotorCount() == 3)
                return;
            if (setStart(4, start4Tb.Text))
            {
                enigma4Tb.Text = start4Tb.Text.ToUpper();
                encrypt();
            }
        }
        private void start1Tb_Leave(object sender, EventArgs e)
        {
            if (setStart(3, start1Tb.Text))
            {
                enigma1Tb.Text = start1Tb.Text.ToUpper();
                encrypt();
            }
        }
        private void start2Tb_Leave(object sender, EventArgs e)
        {
            if (setStart(2, start2Tb.Text))
            {
                enigma2Tb.Text = start2Tb.Text.ToUpper();
                encrypt();
            }
        }
        private void start3Tb_Leave(object sender, EventArgs e)
        {
            if (setStart(1, start3Tb.Text))
            {
                enigma3Tb.Text = start3Tb.Text.ToUpper();
                encrypt();
            }
        }
        private bool setStart(int position, string t)
        {
            Rotor r = _enigma.GetRotor(position);
            try
            {
                r.SetStartPosition(StaticLib.CharToInt26(Convert.ToChar(t)));
                return true;
            }
            catch
            {
                messageTb.Text = (t ?? "[null]") + " is an invalid start setting. "; ;
                return false;
            }
        }

        private void plugsTb_Leave(object sender, EventArgs e)
        {
            _enigma.Plugboard.ClearAllPlugs();

            string[] sArr = plugsTb.Text.Trim().Split(
                new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
            bool error = false;
            int i;
            for (i = 0; i < sArr.Length; i++)
            {
                if (!isValidPlugSet(sArr[i]))
                {
                    error = true;
                    break;
                }
            }
            if (error)
            {
                messageTb.Text = (sArr[i] ?? "[null]") + " is an invalid start setting. ";
                return;
            }
            foreach (string plug in sArr)
            {
                try
                {
                    _enigma.Plugboard.SetPlugs(plug[0], plug[1]);
                }
                catch (Exception ex)
                {
                    messageTb.Text = ex.Message + "\r\nErasing all plugs";
                    plugsTb.Text = "";
                }
            }

            encrypt();
        }

        private bool isValidPlugSet(string ab)
        {
            if (ab == null)
                return false;
            if (ab.Length != 2)
                return false;
            if (ab[0] == ab[1])
                return false;
            try
            {
                int i;
                i = StaticLib.CharToInt26(ab[0]);
                i = StaticLib.CharToInt26(ab[1]);
            }
            catch
            {
                return false;
            }
            return true;
        }

        private void inputTb_TextChanged(object sender, EventArgs e)
        {
            encrypt();
        }

        private void encrypt()
        {
            outputTb.Clear();

            try { _enigma.Reset(); }
            catch { return; }
            foreach (char c in inputTb.Text)
            {
                char printC = ' ';
                try
                {
                    printC = _enigma.Encrypt(c);
                }
                catch { }
                outputTb.AppendText(printC.ToString());
            }
            if (_enigma.GetRotorCount() == 4)
                enigma4Tb.Text = _enigma.GetRotor(4).GetVisibleLetter().ToString();
            enigma1Tb.Text = _enigma.GetRotor(3).GetVisibleLetter().ToString();
            enigma2Tb.Text = _enigma.GetRotor(2).GetVisibleLetter().ToString();
            enigma3Tb.Text = _enigma.GetRotor(1).GetVisibleLetter().ToString();
        }
    }
}