﻿using System;

namespace HistoricalEnigma
{
    public interface IUiHistoricalEnigma
    {
        void ChangeStateToM3Army();
        void ChangeStateToM3Navy();
        void ChangeStateToM4();

        bool Slot4IsVisible();

        void SetRotorInSlot1(EnigmaLib.Rotor rotor);
        void SetRotorInSlot2(EnigmaLib.Rotor rotor);
        void SetRotorInSlot3(EnigmaLib.Rotor rotor);
        void SetRotorInSlot4(EnigmaLib.ThinRotor rotor);
        void SetReflector(EnigmaLib.Reflector reflector);

        int PlugPairCount { get; }
        /// <summary>Get plug pairs</summary>
        /// <returns></returns>
        string[] GetPlugs();
        void SetPlugs(char plug1, char plug2);
        void RemovePlugs(char plug1, char plug2);

        // Enigma UI Members
        /// <summary>Normal = true. Set to false to enter cryptoanalysis mode.</summary>
        bool AutoAdvance { get; set; }
        char Encrypt(char c);
        /// <summary>Get the 1-based position of a rotor</summary>
        /// <param name="rotor"></param>
        /// <returns></returns>
        int GetPosition(EnigmaLib.Rotor rotor);
        /// <summary>Get rotor. 1-based index</summary>
        /// <param name="position"></param>
        EnigmaLib.Rotor GetRotor(int position);
        int GetRotorCount();
        void RemoveRotor(EnigmaLib.Rotor rotor);
        /// <summary>Reset to starting position</summary>
        void Reset();
        /// <summary>Reverse 1 step. Only works if AutoAdvance = true.
        /// Note: reversing won't always result in the letters looking exactly like they
        /// did before the reverse, but since the machine is stepped before encryption,
        /// the encryption will still be correct</summary>
        void Reverse();
        void RemoveReflector();

        // events
        event EnigmaLib.RotorChangedEventHandler RotorAdded;
        event EnigmaLib.RotorChangedEventHandler RotorRemoved;
        event EnigmaLib.ReflectorChangedEventHandler ReflectorAdded;
        event EnigmaLib.ReflectorChangedEventHandler ReflectorRemoved;
        event EventHandler AutoAdvanceChanged;
        event EventHandler RotorCountChanged;

        event EnigmaLib.PlugsChangedEventHandler PlugsAdded;
        event EnigmaLib.PlugsChangedEventHandler PlugsRemoved;

        event EventHandler StateChanged;
    }
}
