using System;
using System.Collections.Generic;

namespace HistoricalEnigma
{
    public enum HistoricalEnigmaStates { M3Army, M3Navy, M4 }

    public abstract class HistoricalEnigmaState
    {
        // number of slots
        public virtual int GetQtyRotors()
        {
            return 3;
        }

        // slot 4 visible?
        public virtual bool Slot4IsVisible()
        {
            return false;
        }

        public abstract List<HistoricalRotors> AvailRotors();

        public virtual List<HistoricalReflectors> AvailReflectors()
        {
            List<HistoricalReflectors> list = new List<HistoricalReflectors>();
            list.Add(HistoricalReflectors.B);
            list.Add(HistoricalReflectors.C);
            return list;
        }
    }

    // Wehrmact (Army)/Luftwaffe (Air Force): M3; UKW B, C; I - V
    public class M3Army : HistoricalEnigmaState
    {
        public override List<HistoricalRotors> AvailRotors()
        {
            List<HistoricalRotors> list = new List<HistoricalRotors>();
            list.Add(HistoricalRotors.I);
            list.Add(HistoricalRotors.II);
            list.Add(HistoricalRotors.III);
            list.Add(HistoricalRotors.IV);
            list.Add(HistoricalRotors.V);
            return list;
        }
    }

    // Kriegsmarine (Navy): M3; UKW B, C; I - VIII
    public class M3Navy : HistoricalEnigmaState
    {
        public override List<HistoricalRotors> AvailRotors()
        {
            List<HistoricalRotors> list = new List<HistoricalRotors>();
            list.Add(HistoricalRotors.I);
            list.Add(HistoricalRotors.II);
            list.Add(HistoricalRotors.III);
            list.Add(HistoricalRotors.IV);
            list.Add(HistoricalRotors.V);
            list.Add(HistoricalRotors.VI);
            list.Add(HistoricalRotors.VII);
            list.Add(HistoricalRotors.VIII);
            return list;
        }
    }

    // Kriegsmarine M4 [4th must be Beta or Gamma]; UKW thin B, thin C; I - VIII, beta, gamma
    // using reflector b is the same as reflector b-thin + last rotor=beta,
    //   set to a, no ring adjustments
    // using reflector c is the same as reflector c-thin + last rotor=gamma,
    //   set to a, no ring adjustments
    public class M4 : HistoricalEnigmaState
    {
        // number of slots
        public override int GetQtyRotors()
        {
            return 4;
        }

        // slot 4 visible?
        public override bool Slot4IsVisible()
        {
            return true;
        }

        public override List<HistoricalRotors> AvailRotors()
        {
            List<HistoricalRotors> list = new List<HistoricalRotors>();
            list.Add(HistoricalRotors.I);
            list.Add(HistoricalRotors.II);
            list.Add(HistoricalRotors.III);
            list.Add(HistoricalRotors.IV);
            list.Add(HistoricalRotors.V);
            list.Add(HistoricalRotors.VI);
            list.Add(HistoricalRotors.VII);
            list.Add(HistoricalRotors.VIII);
            list.Add(HistoricalRotors.Beta);
            list.Add(HistoricalRotors.Gamma);
            return list;
        }

        public override List<HistoricalReflectors> AvailReflectors()
        {
            List<HistoricalReflectors> list = new List<HistoricalReflectors>();
            list.Add(HistoricalReflectors.BThin);
            list.Add(HistoricalReflectors.CThin);
            return list;
        }
    }
}
