using System;

using EnigmaLib;
using HistoricalEnigma;

namespace TestConsole
{
    class Program
    {
        static void Main(string[] args)
        {
            (new Program()).enigmaTest2();
            Console.ReadLine();
        }

        private void enigmaTest1()
        {
            Reflector reflector = HistoryFactory.ReflectorB();
            Rotor i = HistoryFactory.RotorI();
            i.SetStartPosition(7);//H
            Rotor ii = HistoryFactory.RotorII();
            ii.SetStartPosition(3);//D
            Rotor iii = HistoryFactory.RotorIII();
            iii.SetStartPosition(21);//V

            EnigmaMachine e = new EnigmaMachine(3);
            e.SetReflector(reflector);
            e.SetRotor(1, iii);
            e.SetRotor(2, ii);
            e.SetRotor(3, i);
            e.Plugboard.SetPlugs('A', 'E');

            Console.WriteLine(i.Name + "    =" + i.GetVisibleLetter());
            Console.WriteLine(ii.Name + "   =" + ii.GetVisibleLetter());
            Console.WriteLine(iii.Name + "  =" + iii.GetVisibleLetter());
            Console.WriteLine("plugs=" + String.Join("  ", e.Plugboard.GetPlugs()));
            Console.WriteLine();

            for (int j = 0; j < 5; j++)
            {
                char c = e.Encrypt('a');
                Console.WriteLine("a: " + c + "     "
                    + i.GetVisibleLetter() + " " + ii.GetVisibleLetter() + " " + iii.GetVisibleLetter());
            }
            Console.WriteLine();
            for (int j = 0; j < 5; j++)
            {
                e.Reverse();
                Console.WriteLine("reverse: "
                    + i.GetVisibleLetter() + " " + ii.GetVisibleLetter() + " " + iii.GetVisibleLetter());
            }
        }

        private void enigmaTest2()
        {
            Reflector reflector = HistoryFactory.ReflectorB();
            Rotor iii = HistoryFactory.RotorIII();
            iii.SetStartPosition(23);
            iii.SetRing('N');
            Rotor v = HistoryFactory.RotorV();
            v.SetStartPosition(14);
            v.SetRing('A');
            Rotor ii = HistoryFactory.RotorII();
            ii.SetStartPosition(12);
            ii.SetRing('S');

            EnigmaMachine e = new EnigmaMachine(3);
            e.SetReflector(reflector);
            e.SetRotor(1, ii);
            e.SetRotor(2, v);
            e.SetRotor(3, iii);
            e.Plugboard.SetPlugs('Q', 'G');
            e.Plugboard.SetPlugs('A', 'I');
            e.Plugboard.SetPlugs('W', 'N');
            e.Plugboard.SetPlugs('S', 'B');
            e.Plugboard.SetPlugs('E', 'U');
            e.Plugboard.SetPlugs('X', 'L');
            e.Plugboard.SetPlugs('D', 'Z');
            e.Plugboard.SetPlugs('O', 'R');

            Console.WriteLine(iii.Name + " start:" + iii.GetVisibleLetter() + " ring:N");
            Console.WriteLine(v.Name + "   start:" + v.GetVisibleLetter() + " ring:A");
            Console.WriteLine(ii.Name + "  start:" + ii.GetVisibleLetter() + " ring:S");
            Console.WriteLine("plugs=" + String.Join("  ", e.Plugboard.GetPlugs()));
            Console.WriteLine();

            encrypt(e, "DinahIsTheCoolestSpy");
            e.Reset();
            Console.WriteLine();
            encrypt(e, "ZWZQPKUNURNLEFAIAMIZ");
        }

        private void enigmaTest3()
        {
            Reflector reflector = HistoryFactory.ReflectorB();
            Rotor i = HistoryFactory.RotorI();
            i.SetStartPosition(7);//H
            Rotor ii = HistoryFactory.RotorII();
            ii.SetStartPosition(3);//D
            Rotor iii = HistoryFactory.RotorIII();
            iii.SetStartPosition(21);//V

            EnigmaMachine e = new EnigmaMachine(3);
            e.SetReflector(reflector);
            e.SetRotor(1, iii);
            e.SetRotor(2, ii);
            e.SetRotor(3, i);
            e.Plugboard.SetPlugs('A', 'E');

            Console.WriteLine(i.Name + "    =" + i.GetVisibleLetter());
            Console.WriteLine(ii.Name + "   =" + ii.GetVisibleLetter());
            Console.WriteLine(iii.Name + "  =" + iii.GetVisibleLetter());
            Console.WriteLine("plugs=" + String.Join("  ", e.Plugboard.GetPlugs()));
            Console.WriteLine();

            encrypt(e, "HelloJessica");
            e.Reset();
            Console.WriteLine();
            encrypt(e, "SPVYJZKCNUOT");
        }
        private void encrypt(EnigmaMachine enigma, string message)
        {
            foreach (char c in message)
            {
                char cOut = enigma.Encrypt(c);
                Console.WriteLine(c.ToString() + ": " + cOut.ToString());
            }
        }
    }
}
